import logging
import pathlib

from configparser import ConfigParser

from typing import List

from ec2init import Procedure
from ec2init.utils.system import SystemUtils

logger = logging.getLogger(__name__)

class InitHopsCA(Procedure):
    def __init__(self, name: str, config: ConfigParser) -> None:
        self.name = name
        self.config = config

    def pre(self) -> None:
        if self.report_callback is not None:
            try:
                self.report_callback.execute()
            except Exception as ex:
                logger.warning("%s - Could not execute pre-execute: %s", self.name, ex)

    def execute(self) -> List[Procedure]:
        installation_dir = self.config.get('general', 'installation_dir', fallback='/root/ec2init')
        init_ca_script = pathlib.Path(installation_dir, "init_hops-ca.sh")
        logger.info("%s - Initializing Hops CA", self.name)
        SystemUtils.execute_command(["/bin/bash", str(init_ca_script)], timeout=120)
        logger.info("%s - Finished initializing Hops CA", self.name)
        return []

    def rollback(self) -> None:
        pass