from ec2init import Procedure

from configparser import ConfigParser
from pathlib import Path
from typing import List, Dict
import logging
from ec2init.plans.multinode.generate_service_jwt import BaseGenerateServiceJWT
from ec2init.utils.http import HttpUtils
from ec2init.utils.database import databaseUtils
import yaml 
from yaml.loader import SafeLoader

logger = logging.getLogger(__name__)

class GenerateElasticSigningKey(Procedure):
    
    GEN_ELK_TOKEN_URI = ("GET", "https://hopsworks.glassfish.service.consul:8182/hopsworks-api/api/jwt/elk/key")

    def __init__(self, name: str, config: ConfigParser, check_file: Path) -> None:
        self.name = name
        self.config = config
        self.check_file = check_file

    def execute(self) -> List[Procedure]:
        logger.info("%s - Starting execution", self.name)
        signing_key = self._get_elk_signing_key()
        config_yml = "/srv/hops/elasticsearch/plugins/opendistro_security/securityconfig/config.yml"
        with open(config_yml, 'r') as f:
            config_dict : Dict  = yaml.load(f, Loader=SafeLoader)
        config_dict['config']['dynamic']['authc']['jwt_auth_domain']['http_authenticator']['config']['signing_key'] = signing_key
        with open(config_yml, 'w') as f:
            yaml.dump(config_dict, f, sort_keys=False, default_flow_style=False)
        
        if self.check_file is not None:
            self.check_file.touch()
        logger.info("%s - Finished execution", self.name)
        return []

    def _get_elk_signing_key(self) -> str:
        databaseUtils.run_delete_statement("jwt_signing_key", "name = 'elk_jwt_signing_key'")

        master_token, _ = BaseGenerateServiceJWT.get_service_jwt_tokens(self.config)
        headers = {
            'Content-Type': 'application/json',
            'Authorization': f"Bearer {master_token}"
        }

        method, uri = GenerateElasticSigningKey.GEN_ELK_TOKEN_URI

        response: Dict[str, str] = HttpUtils.do_plain_call(self.config, uri, method, headers=headers, verify=False)
        return response['response']


    def rollback(self) -> None:
        pass
