import logging
import time

from typing import List

from ec2init import Procedure
from ec2init.plans.init.start_services import StartServices
from ec2init.utils.system import SystemUtils

logger = logging.getLogger(__name__)

class StartSideServices(Procedure):

    SIDE_CAR_SERVICES = ["node_exporter", "kagent", "filebeat-service"]

    def __init__(self, name: str) -> None:
        self.name = name

    def pre(self) -> None:
        pass

    def rollback(self) -> None:
        pass

    def execute(self) -> List[Procedure]:
        logger.info(f"{self.name} - Starting execution")

        for service in StartSideServices.SIDE_CAR_SERVICES:
            proceed = True
            if service == "node_exporter":
                proceed = StartServices.is_service_installed("node_exporter")
            
            if proceed:
                logger.info(f"{self.name} - Starting {service}")
                SystemUtils.execute_command(StartServices.SYSTEMCTL_ENABLE.format(service=service).split(" "))
                SystemUtils.execute_command(StartServices.SYSTEMCTL_START.format(service=service).split(" "), timeout=120)
                logger.info(f"{self.name} - Started {service}")

        time.sleep(5)
        return []

    @staticmethod
    def has_started_services() -> bool:
        for service in StartSideServices.SIDE_CAR_SERVICES:
            proceed = True
            if service == "node_exporter":
                proceed = StartServices.is_service_installed("node_exporter")
            if proceed:
                if not StartServices.is_service_enabled(service) or not StartServices._is_service_running(service):
                    logger.warn(f"Service {service} is not active/running")
                    return False
        return True
