from typing import Dict, List, Optional, cast
from ec2init.utils.http import HttpUtils
import subprocess
from ec2init.utils.system import SystemUtils
from configparser import ConfigParser

MYSQL_CLIENT = "/srv/hops/mysql-cluster/ndb/scripts/mysql-client.sh"
LOGIN_URI = (
    "POST", "https://hopsworks.glassfish.service.consul:8182/hopsworks-api/api/auth/login")
REFRESH_VARIABLE_URI = (
    "POST", "https://hopsworks.glassfish.service.consul:8182/hopsworks-api/api/admin/variables/refresh")

class databaseUtils:

    ADMIN_EMAIL = "admin@hopsworks.ai"

    @staticmethod
    def _login(user_email: str, user_password: Optional[str], config: ConfigParser):
        if not user_email or not user_password:
            raise Exception("Admin username or password is not set")

        method, uri = LOGIN_URI
        data = {
            'email': user_email,
            'password': user_password,
        }
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }

        response: Dict[str, str] = HttpUtils.do_plain_call(
            config, uri, method, data=data, headers=headers, verify=False)
        response_headers: Dict[str, str] = cast(
            Dict[str, str], response['headers'])

        jwt = response_headers['Authorization']
        return jwt

    @staticmethod
    def refresh_variables(config: ConfigParser, jwt: str):

        method, uri = REFRESH_VARIABLE_URI

        headers = {
            'Content-Type': 'application/json',
            'Authorization': jwt
        }

        HttpUtils.do_api_call(config, uri, method, headers=headers, verify=False)
        
    @staticmethod
    def sysexec(cmd: List[str], timeout: int = 120) -> subprocess.CompletedProcess:
        return SystemUtils.execute_command(cmd, timeout=timeout)

    @staticmethod
    def update_variables_stmt(id: str, value: str) -> str:
        return f"UPDATE variables SET value='{value}' WHERE id='{id}'"

    @staticmethod
    def update_variables_db_cmd(id: str, value: str) -> List[str]:
        return [MYSQL_CLIENT, "hopsworks", "-e", databaseUtils.update_variables_stmt(id, value)]

    @staticmethod
    def update_variable(id: str, value: str):
        databaseUtils.sysexec(databaseUtils.update_variables_db_cmd(id, value))

    @staticmethod
    def run_command(cmd: List[str]) -> subprocess.CompletedProcess:
        cmd.insert(0, MYSQL_CLIENT)
        return databaseUtils.sysexec(cmd)

    @staticmethod
    def insert_variables_db_cmd(id: str, value: str, visibility: int = 0) -> List[str]:
        return [MYSQL_CLIENT, "hopsworks", "-e", databaseUtils.insert_variables_stmt(id, value, visibility)]

    @staticmethod
    def insert_variables_stmt(id: str, value: str, visibility: int = 0) -> str:
        return f"INSERT INTO variables (id, value, visibility) VALUES ('{id}', '{value}', {visibility})"

    @staticmethod
    def run_delete_statement(table: str, condition: str) -> None:
        databaseUtils.sysexec([MYSQL_CLIENT, "hopsworks", "-e", f"DELETE from {table} where {condition}"])
